package main

import (
	"fmt"
	"time"
)

func main() {
	intChan := make(chan int)
	strChan := make(chan string, 3)

	fmt.Println(len(intChan))
	fmt.Println(len(strChan))

	// 赋值，必须放到 goroutine 中赋值
	go func(intChan chan int, strChan chan string) {
		intChan <- 8
		time.Sleep(2 * time.Second)
	}(intChan, strChan)

	// 取值
	fmt.Println(len(intChan))
	fmt.Println(<-intChan)

	strChan <- "hello chan"
	fmt.Println(len(strChan))
	strChan <- "hello chan"
	// 关闭Chan，关闭后只能读，不能写
	close(strChan)
	//strChan <- "hello chan" // panic: send on closed channel

	fmt.Println(<-strChan)
	v, ok := <-strChan
	fmt.Printf("%T %#v, %T %#v\n", v, v, ok, ok)
	fmt.Println(len(strChan))

	// 遍历管道，只有 close(strChan) 时，才能使用 for range，否则会发生死锁
	for s := range strChan {
		fmt.Println(s)
	}

	// Output:
	//0
	//0
	//0
	//8
	//1
	//hello chan
	//string "hello chan", bool true
	//0
}
